/*
 * ECRTS 2010 submission "Sulphur"
 * main program
 * Copyright (C) Jack Whitham 2010
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#include "tetra.h"
#include "smmu.h"
#include "cache.h"
#include "memlib.h"

#include "u64.hh"

#define NUM_TESTS       1000
#define NUM_SIZES       9
#define NUM_LATENCIES   3

static ulongint on_chip_size [ NUM_SIZES ] = {
        16384, 512, 1024, 2048, 
        4096, 8192, 32768, 65536, 
        131072 };
static ulongint bus_latency [ NUM_LATENCIES ] = {
        49, 24, 98 };


int main (int argc, char ** argv)
{
    ulongint        i, j, k;
    ulongint        num_tests;
    int             prof;
    FILE *          data = fopen("memlab_cdata.py", "wt");

    assert (data != NULL);

    prof = (argc > 1) && (strcmp(argv[ 1 ], "prof") == 0);
    num_tests = prof ? 2 : NUM_TESTS;

    printf("Running memlab%s\n", prof ? " (profiling mode)" : "");

    for (i = 0; tetra_methods[ i ] != NULL; i++) {
        Tetra_Params *  tp;
        const char *    name = tetra_descs[ i ];
        ulongint        cache_acet[ NUM_SIZES ];
        ulongint        smmu_ro_wcet[ NUM_SIZES ];
        ulongint        smmu_1, smmu_2;

        assert (name != NULL);
        printf ( "%s: ", name) ;
        fflush(stdout);
        tp = Init_Params ();
        tp->buslat = bus_latency[ 0 ];
        for (k = 0; k < NUM_SIZES; k ++) {
            tp->bytes_on_chip = on_chip_size[ k ];
            Init_Cache(tp, 64);
            tp->clock = 0;
            for (j = 0; j < num_tests; j ++) {
                tetra_methods[ i ](tp);
                tp->flush_fn(tp->mem_model_cb);
            }
            tp->destroy_fn(tp->mem_model_cb);
            cache_acet[ k ] = tp->clock / num_tests;
            fprintf(data, "data[ ('%s', 'Cache_%"FU"', %"FU") ] = %"FU".0\n",
                    name, tp->bytes_on_chip, 
                    tp->buslat, tp->clock / num_tests);
            printf ( "." );
            fflush ( stdout );
            fflush ( data );
        }

        tp->bytes_on_chip = on_chip_size[ 0 ];
        tp->force_worst = 1;
        tp->clock = 0;
        Init_SMMU(tp, 16, CM_ORIGINAL);
        tetra_methods[ i ](tp);
        tp->flush_fn(tp->mem_model_cb);
        tp->destroy_fn(tp->mem_model_cb);
        smmu_1 = tp->clock;
        fprintf(data, "data[ ('%s', 'SMMU_F', %"FU") ] = %"FU".0\n",
                name, tp->buslat, tp->clock);
        printf ( "." );
        fflush ( stdout );

        tp->tiling_flag = 1;
        tp->clock = 0;
        Init_SMMU(tp, 16, CM_ORIGINAL);
        tetra_methods[ i ](tp);
        tp->flush_fn(tp->mem_model_cb);
        tp->destroy_fn(tp->mem_model_cb);
        smmu_2 = tp->clock;
        fprintf(data, "data[ ('%s', 'SMMU_TF', %"FU") ] = %"FU".0\n",
                name, tp->buslat, tp->clock);
        printf ( "." );
        fflush ( stdout );

        tp->tiling_flag = 1;
        for (k = 0; k < NUM_SIZES; k ++) {
            tp->bytes_on_chip = on_chip_size[ k ];
            tp->clock = 0;
            Init_SMMU(tp, 16, CM_OPTION_B);
            tetra_methods[ i ](tp);
            tp->flush_fn(tp->mem_model_cb);
            tp->destroy_fn(tp->mem_model_cb);
            smmu_ro_wcet[ k ] = tp->clock;
            fprintf(data, "data[ ('%s', 'SMMU_%"FU"', %"FU") ] = %"FU".0\n",
                    name, tp->bytes_on_chip, tp->buslat, tp->clock);
            printf ( "." );
            fflush ( stdout );
            fflush ( data );
        }

        tp->bytes_on_chip = on_chip_size[ 0 ];
        for (k = 1; k < NUM_LATENCIES; k ++) {
            tp->buslat = bus_latency[ k ];
            tp->tiling_flag = 1;
            tp->force_worst = 1;
            tp->clock = 0;
            Init_SMMU(tp, 16, CM_OPTION_B);
            tetra_methods[ i ](tp);
            tp->flush_fn(tp->mem_model_cb);
            tp->destroy_fn(tp->mem_model_cb);
            fprintf(data, "data[ ('%s', 'SMMU_%"FU"', %"FU") ] = %"FU".0\n",
                    name, tp->bytes_on_chip, tp->buslat, tp->clock);
            printf ( "." );
            fflush ( stdout );

            Init_Cache(tp, 64);
            tp->force_worst = 0;
            tp->clock = 0;
            for (j = 0; j < num_tests; j ++) {
                tetra_methods[ i ](tp);
                tp->flush_fn(tp->mem_model_cb);
            }
            tp->destroy_fn(tp->mem_model_cb);
            fprintf(data, "data[ ('%s', 'Cache_%"FU"', %"FU") ] = %"FU".0\n",
                    name, tp->bytes_on_chip, 
                    tp->buslat, tp->clock / num_tests);
            printf ( "." );
            fflush ( stdout );
            fflush ( data );
        }
        printf ( "\n%s %f %f %f\n" , name,
                ((double) cache_acet[ 0 ]) / smmu_1,
                ((double) cache_acet[ 0 ]) / smmu_2,
                ((double) cache_acet[ 0 ]) / ((double) smmu_ro_wcet[ 0 ]));
        fflush ( stdout );
        free(tp);
    }
    fclose(data);
    return 0;
}


